/**
 * @file servicelog_manage.c
 * @brief The servicelog_manage utility.
 *
 * Copyright (C) 2006 IBM Corporation
 *
 * @author Michael Strosaker <strosake@us.ibm.com>
 * @author Nathan Fontenot <nfont@austin.ibm.com>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#define _GNU_SOURCE
#include <getopt.h>
#include <servicelog-1/servicelog.h>

#define ACTION_TOOMANY		-1
#define ACTION_UNSPECIFIED	0
#define ACTION_STATUS		1
#define ACTION_EVENTS		2
#define ACTION_NOTIFY		3
#define ACTION_TRUNCATE_EVENTS	4
#define ACTION_TRUNCATE_NOTIFY	5
#define ACTION_CLEAN		6

#define ARG_LIST	"st:fhq"

#define SECONDS_IN_DAY		24 * 60 * 60
#define SECONDS_IN_YEAR		365 * SECONDS_IN_DAY

static char *cmd;

static struct option long_options[] = {
	{"status",	    no_argument,        NULL, 's'},
	{"truncate",	required_argument,  NULL, 't'},
	{"force",	    no_argument,	NULL, 'f'},
	{"quiet",	    no_argument,	NULL, 'q'},
	{"help",	    no_argument,        NULL, 'h'},
	{0,0,0,0}
};

/**
 * print_usage
 * @brief Print the usage message
 */
static void
print_usage()
{
	printf("Usage:\n");
	printf("  %s --status       return status\n", cmd);
	printf("  %s --truncate events   delete all events\n", cmd);
	printf("  %s --truncate notify   delete all notification tools\n", cmd);

	printf("  Other Flags:\n");
	printf("    --help             print this help text and exit\n");
	printf("    --force            do not prompt the user to verify\n");

	return;
}

/**
 * main
 * @brief Parse command line args process database
 *
 * @param argc the number of command-line arguments
 * @param argv array of command-line arguments
 * @return exit status: 0 for normal exit, 1 for usage error, >1 for other error
 */
int
main(int argc, char *argv[])
{
	struct servicelog *slog;
	int rc = 0;
	struct sl_event *event;
	struct sl_repair_action *repairs;
	struct sl_notify *notifications;
	int option_index, action=ACTION_UNSPECIFIED;
	int flag_force=0, quiet = 0;
	char buf[124];
	char *tmp;
	uint32_t num=0, num_repaired=0, num_unrepaired=0, num_info=0, num_ra=0;

	if (argc <= 1) {
		print_usage();
		exit(0);
	}

	memset(&slog, 0, sizeof(slog));

	for (;;) {
		option_index = 0;
		rc = getopt_long(argc, argv, ARG_LIST, long_options,
				 &option_index);

		if (rc == -1)
			break;

		switch (rc) {
		case 's':
			if (action != ACTION_UNSPECIFIED)
				action = ACTION_TOOMANY;
			if (action != ACTION_TOOMANY)
				action = ACTION_STATUS;
			break;
		case 't':
			if (!optarg) {
				fprintf(stderr, "The --truncate option "
					"requires either \"events\" or "
					"\"notify\" as an argument.\n");
				print_usage(argv[0]);
				exit(-1);
			}

			if (!strncmp(optarg, "events", strlen(optarg))) {
				if (action != ACTION_UNSPECIFIED)
					action = ACTION_TOOMANY;
				if (action != ACTION_TOOMANY)
					action = ACTION_TRUNCATE_EVENTS;
			}
			else if (!strncmp(optarg, "notify", strlen(optarg))) {
				if (action != ACTION_UNSPECIFIED)
					action = ACTION_TOOMANY;
				if (action != ACTION_TOOMANY)
					action = ACTION_TRUNCATE_NOTIFY;
			}
			else {
				fprintf(stderr, "The --truncate option "
					"requires either \"events\" or "
					"\"notify\" as an argument.\n");
				print_usage(argv[0]);
				exit(-1);
			}
			break;
		case 'f':
			flag_force = 1;
			break;
		case 'q':
			quiet = 1;
			break;
		case 'h':	/* help */
			print_usage(argv[0]);
			exit(0);
			break;
		case '?':
			print_usage(argv[0]);
			exit(1);
			break;
		default:
			printf("Invalid argument: %s\n", optarg);
		}
	}

	/* Command-line validation */
	if (action == ACTION_UNSPECIFIED) {
		fprintf(stderr, "One of the action options is required.\n");
		print_usage(argv[0]);
		exit(-1);
	}

	if (action == ACTION_TOOMANY) {
		fprintf(stderr, "Only one of the action options may be "
			"specified.\n");
		print_usage(argv[0]);
		exit(-1);
	}


	switch (action) {

	case ACTION_STATUS:
		rc = servicelog_open(&slog, 0);
		if (rc != 0) {
			if (!quiet)
				fprintf(stderr, "%s: Could not open servicelog "
					"database to log the repair action.\n%s\n",
					argv[0], servicelog_error(slog));
			return 2;
		}

		rc = servicelog_event_query(slog, "", &event);
		if (rc != 0) {
				fprintf(stderr, "%s\n", servicelog_error(slog));
				exit(2);
		}

		 while (event) {
			 num++; // total event count
			 if (event->serviceable && (event->repair > 0))
				 num_repaired++;
			 else if (event->serviceable)
				 num_unrepaired++;
			 else
				 num_info++; // informational events
			 event = event->next;
		 }

		servicelog_event_free(event);

		// Now need to query repair actions:
		servicelog_repair_query(slog, "", &repairs);

		while (repairs) {
			num_ra++;
			repairs = repairs->next;
		}
		servicelog_repair_free(repairs);

		servicelog_close(slog);


		printf("%-39s%10u\n", "Logged events:", num);
		printf("    %-35s%10u\n", "unrepaired serviceable events:",
		       num_unrepaired);
		printf("    %-35s%10u\n", "repaired serviceable events:",
		       num_repaired);
		printf("    %-35s%10u\n", "informational events:", num_info);
		printf("    %-35s%10u\n", "repair actions:", num_ra);

		break;


	case ACTION_TRUNCATE_EVENTS:
		if (geteuid() != 0) // Check to see if user is root
		{
			printf("Must be root to truncate the database!\n");
			exit(-1);
		}

		num = 0;

		if (!flag_force) {
			printf("Are you certain you wish to delete ALL events "
			       "from the servicelog?\n");
			printf("Enter 'yes' to continue > ");
			tmp = fgets(buf, 80, stdin);
			if (strcasecmp(buf, "yes\n")) {
				printf("Operation cancelled.\n");
				break;
			}
		}

		rc = servicelog_open(&slog, SL_FLAG_ADMIN);
		if (rc != 0) {
			if (!quiet)
				fprintf(stderr, "%s: Could not open servicelog "
					"database to log the repair action.\n%s\n",
					argv[0], servicelog_error(slog));
			return 2;
		}
		rc = servicelog_event_query(slog, "", &event);
		if (rc != 0) {
				fprintf(stderr, "%s\n", servicelog_error(slog));
				exit(2);
		}

		 while (event) {
			 num++;
			 servicelog_event_delete(slog, event->id);
			 event = event->next;
		 }


		printf("Deleted %u records.\n", num);
		servicelog_close(slog);

		rc = 0;
		break;

	case ACTION_TRUNCATE_NOTIFY:
		if (geteuid() != 0) // Check to see if user is root
		{
			printf("Must be root to truncate the database!\n");
			exit(-1);
		}

		num = 0;

		if (!flag_force) {
			printf("Are you certain you wish to delete ALL "
					"notification tools from the servicelog?\n");
			printf("Enter 'yes' to continue > ");
			tmp = fgets(buf, 80, stdin);
			if (strcasecmp(buf, "yes\n")) {
				printf("Operation cancelled.\n");
				break;
			}
		}

		rc = servicelog_open(&slog, SL_FLAG_ADMIN);
		if (rc != 0) {
			if (!quiet)
				fprintf(stderr, "%s: Could not open servicelog "
					"database to log the repair action.\n%s\n",
					argv[0], servicelog_error(slog));
			return 2;
		}
		rc = servicelog_notify_query(slog, "", &notifications);
		if (rc != 0) {
				fprintf(stderr, "%s\n", servicelog_error(slog));
				exit(2);
		}

		 while (notifications) {
			 num++;
			 servicelog_notify_delete(slog, notifications->id);
			 notifications = notifications->next;
		 }
		servicelog_notify_free(notifications);
		servicelog_close(slog);

		printf("Deleted %u records.\n", num);
		rc = 0;
		break;

	default:
		fprintf(stderr, "Internal error; unknown action %d\n", action);
		rc = -1;
	}

	return rc;
}
